/* ***************************************************************************
 *
 *	This program is free software; you can redistribute it and/or modify it
 * 	under the terms of the Shared Code License (SCL) version 1.0 as published
 *	by www.IntellectualHeaven.com. You should have received a copy of the SCL
 *	1.0 license, look for file license.txt, along with this software; if not,
 *	then please write to x_pankaj_x@intellectualheaven.com for an electronic
 *	copy of the same.
 *
 * File name:
 * 		SfCui.cpp
 * 
 * Description:
 * 		This file implements the main entry point for console SnifFire app
 * 	
 * Revision:
 * --------------------------------------------------------------------------
 *		2005-04-24	xpankajx@intellectualheaven.com
 *		First version
 * --------------------------------------------------------------------------
 * 	Copyright (c) : Pankaj Garg
 * **************************************************************************/

//System-Includes!!!
#include <windows.h>
#include <stdio.h>
#include <string>

// Local-Includes!!!
#include "sfcuiview.h"

/*
 * Our Application's Title
 */
#define SFCUI_APP_TITLE		L"SnifFire - Ethernet Sniffer + Pseudo TCP/IP Firewall"


/*
 * Global color variable to record color on application start. This variable
 * is used to restore original color on Ctrl-C or normal application exit
 */
WORD gDefaultColor;


/*
 * Global variable to record the title of cmd window on application start.
 * This variable is used to restore original color on Ctrl-C or normal
 * application exit
 */
wchar_t gConsoleTitle[MAX_PATH];


/*
 * Global variables to track the state of driver installation and sniffer
 */
bool gDriverInstalled = false;
bool gSnifferActivated = false;



/****************************************************************************
 *  FUNCTION NAME:  sfProcessExit
 *
 *  DESCRIPTION:    This routine handles the CTRL-C, CTRL-BREAK etc. events.
 *					On such event it removes the IAT patches from the process
 *					and then return FALSE to invoke system Ctrl handler.
 *
 *  INPUTS:         inCtrlType - A well defined ctrl type code. We don't use it
 *
 *  RETURNS:		TRUE means we handled the code. FALSE means we didn't
 *					handle and it causes the application to be killed
 ***************************************************************************/
BOOL
WINAPI
SfProcessExit(
	DWORD inCtrlType)
{
	// restore the cmd window color
	SfUtilChangeTextClr(gDefaultColor);

	if (gSnifferActivated)
	{
		wprintf(L"Stopping the Sniffer.\n");
		SetFilterDirected();
		DeactivateSniffer();
	}

	// Deinitialize the driver
	if (gDriverInstalled)
	{
		wprintf(L"Removing the RawNetIo driver from the system.\n");
		RnIf_DriverExitInstance();
		wprintf(L"RawNetIo driver removed.\n");
	}

	wprintf(L"Terminating application.\n");

	// restore the cmd window title
	SetConsoleTitle(gConsoleTitle);

	return FALSE;
}



/****************************************************************************
 *  FUNCTION NAME:  wmain
 *
 *  DESCRIPTION:    Application entry point
 ***************************************************************************/
int
__cdecl
wmain(int argC, wchar_t *argV[])
{
	ULONG		adapterIndex			= 0;
	wchar_t		userAnswer				= L'N';
	wchar_t		deniedIpFile[MAX_PATH]	= {0};
	CSfCuiView	cuiView;

	SfInitialize(&cuiView);
	wprintf(L"\n");
	
	// save the original window title and set it to ours
	GetConsoleTitle(gConsoleTitle, MAX_PATH - 1);
	SetConsoleTitle(SFCUI_APP_TITLE);

	// Change the foreground color
	gDefaultColor = SfUtilChangeTextClr(FOREGROUND_WHITE);

	// set ctrl-c/break handler
	SetConsoleCtrlHandler(SfProcessExit, TRUE);
	
	wprintf(L"Installing a new instance of RawNetIo driver.\n");

	// Initialize the RawNetIo driver
	if (RnIf_DriverInitInstance(L"RawNetCui") == false)
	{
		wprintf(L"Failed to install the RawNetIo driver. Error: %x\n",
				GetLastError());

		gDriverInstalled = false;
		goto funcExit;
	}

	wprintf(L"A new instance of RawNetIo driver installed.\n");
	gDriverInstalled = true;
	
	// Enumerate the adapters
	RNIF_ADAPTER_INFO adapterArray[64];
	RNIF_ADAPTER_INFO_LIST adapterInfoList;
	adapterInfoList.AdapterArray = adapterArray;
	adapterInfoList.MaximumCount = 64;
	adapterInfoList.CurrentCount = 0;
	RnIf_AdapterEnumerate(adapterInfoList);
	ULONG numAdapters = adapterInfoList.CurrentCount;

	if (numAdapters == 0)
	{
		wprintf(L"You have no qualified adapters to bind.\n");
		goto funcExit;
	}

	wprintf(L"-------------------------------------------------------------\n");
	wprintf(L"Adapter |  Adapter Name\n");
	wprintf(L"Number  |\n");
	wprintf(L"--------|----------------------------------------------------\n");
	for (ULONG i = 0; i < numAdapters; ++i)
	{
		wprintf(L"   %2d   |  %ws\n", 
				i + 1,
				adapterInfoList.AdapterArray[i].FriendlyName);
	}
	wprintf(L"--------|----------------------------------------------------\n");
	wprintf(L"Please type an adapter number or 0 (to exit): ");
	fflush(stdin);
	wscanf(L"%d", &adapterIndex);

	if (adapterIndex == 0 || adapterIndex > numAdapters)
	{
		goto funcExit;
	}

	fflush(stdin);
	wprintf(L"Would you like to activate TCP firewall(Y/N): ");
	userAnswer = getwc(stdin);

	if (userAnswer == L'Y' || userAnswer == L'y')
	{
		wprintf(L"Enter the blocked IP address file name: ");
		fflush(stdin);
		fgetws(
			deniedIpFile,
			(sizeof(deniedIpFile) / sizeof(wchar_t)) - 1,
			stdin);

		deniedIpFile[(sizeof(deniedIpFile) / sizeof(wchar_t)) - 1] = L'\0';
		int nLength = wcslen(deniedIpFile);
		if (deniedIpFile[nLength - 1] == L'\n')
		{
			deniedIpFile[nLength - 1] = L'\0';
		}

		if (ActivateFirewall(deniedIpFile) == false)
		{
			wprintf(L"Failed to open the file. Firewall is not enabled.\n");
		}
		else
		{
			wprintf(L"Firewall is activated. All the IP addresses in the given "
					L"file are added to blocked IP list.\n");
		}
	}

	wprintf(L"Activating sniffer on adapter:\n");

	SfUtilChangeTextClr(FOREGROUND_GREEN | FOREGROUND_INTENSITY);

	wprintf(L"%s\n",
			adapterInfoList.AdapterArray[adapterIndex - 1].FriendlyName);

	SfUtilChangeTextClr(
				FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_INTENSITY);

	wprintf(L"Press Ctrl-C to exit.\n");

	SfUtilChangeTextClr(FOREGROUND_WHITE);

	Sleep(3000);

	if (ActivateSniffer(adapterInfoList.AdapterArray[adapterIndex - 1]))
	{
		gSnifferActivated = true;
		SetFilterPromiscuous();
	}

	/*
	 * Just keep rolling in an infinite loop until user press Ctrl-C to cancel
	 * it and terminate the app
	 */
	while (true)
	{
		Sleep(1000);
	}

funcExit:

	SfProcessExit(0);

	return 0;
}

