/* ***************************************************************************
 *
 *	This program is free software; you can redistribute it and/or modify it
 * 	under the terms of the Shared Code License (SCL) version 1.0 as published
 *	by www.IntellectualHeaven.com. You should have received a copy of the SCL
 *	1.0 license, look for file license.txt, along with this software; if not,
 *	then please write to x_pankaj_x@intellectualheaven.com for an electronic
 *	copy of the same.
 *
 * File name:
 * 		SfGuiView.cpp
 * 
 * Description:
 * 		This file implements the bulk of GUI view functionality for the 
 *		application. It handles everything from Window creation, to menu
 *		handling and packet display.
 * 	
 * Revision:
 * --------------------------------------------------------------------------
 *		2005-04-24	xpankajx@intellectualheaven.com
 *		First version
 * --------------------------------------------------------------------------
 * 	Copyright (c) : Pankaj Garg
 * **************************************************************************/
 
// System-Includes!!!
#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <stdlib.h>
#include <commdlg.h>


// Local-Includes!!!
#include "rawnetif.h"
#include "Sf.h"
#include "SfGuiTypes.h"
#include "SfGui.h"
#include "SfGuiView.h"
#include "SfGuiRes.h"


/*
 * Main list control header headings and their width and
 * their horizontal position in the list control
 */
#define SFGUI_SOURCE_MAC_ADDRESS_POS		0
#define SFGUI_DEST_MAC_ADDRESS_POS		1
#define SFGUI_DATAGRAM_NAME_POS			2
#define SFGUI_SOURCE_IP_ADDRESS_POS		3
#define SFGUI_DEST_IP_ADDRESS_POS			4

ListControlHeaderData gMainListControlData[] = 
	{
		{ 150,	L"Source MAC Address",		SFGUI_SOURCE_MAC_ADDRESS_POS },
		{ 150,	L"Destination MAC Address",	SFGUI_SOURCE_MAC_ADDRESS_POS },
		{ 100,	L"Datagram",				SFGUI_DATAGRAM_NAME_POS },
		{ 140,	L"Source IP Address",		SFGUI_SOURCE_IP_ADDRESS_POS },
		{ 140,	L"Destination IP Address",	SFGUI_DEST_IP_ADDRESS_POS }
	};



/*
 * Dialog Proc for Adapter List Dialog
 */
INT_PTR
CALLBACK
AdapterListDlgProc(	
				HWND hDlg,
				UINT msg,
				WPARAM wParam,
				LPARAM lParam);



/****************************************************************************
 *  FUNCTION NAME:  CSfGuiView
 *
 *  DESCRIPTION:    Constructor
 *
 *  NOTES:          
 *
 ***************************************************************************/
CSfGuiView::CSfGuiView(HINSTANCE hInstance)
{
    m_hInstance = hInstance;

	m_SnifferActivated		= false;
	m_ProtectingMyComputer	= false;
	m_FirewallActivated		= false;
	m_NumListElements		= 0;
	ghAdapter				= INVALID_HANDLE_VALUE;
    
	// Common controls must be initialized
    InitCommonControls();
}



/****************************************************************************
 *  FUNCTION NAME:  ~CSfGuiView
 *
 *  DESCRIPTION:    Destructor
 *
 *  NOTES:          
 *
 ***************************************************************************/
CSfGuiView::~CSfGuiView()
{
}



/****************************************************************************
 *  FUNCTION NAME:  WndProc
 *
 *  DESCRIPTION:    Windows main message handling function
 *
 *  INPUTS:         hwnd - Handle to associated window
 *                  nMsg - Message to Handle
 *                  wParam - 
 *                  lParam - 
 *
 *  RETURNS:        LRESULT
 *
 *  NOTES:          
 *
 ***************************************************************************/
LRESULT
CSfGuiView::WndProc(	HWND hwnd,
                        UINT nMsg,
                        WPARAM wParam,
                        LPARAM lParam)
{
    switch(nMsg)
    {
        case WM_NOTIFY:
        {
            LPNMHDR		pNM		= (LPNMHDR)lParam;

			/* 
             * handle custom draw notifications
             */
            if(pNM->hwndFrom == m_hLVMain)
            {
				// This item will be used to calculate the selectedItem
				int nSelectedItem = 0;

				switch(pNM->code)
				{
					case NM_CUSTOMDRAW:
					{
						/*
						* Custom draw from list view
						*/
						LPNMLVCUSTOMDRAW pCD = (LPNMLVCUSTOMDRAW)lParam;

						if(pCD->nmcd.dwDrawStage == CDDS_PREPAINT)
						{
							return CDRF_NOTIFYITEMDRAW;
						}
						else if(pCD->nmcd.dwDrawStage == CDDS_ITEMPREPAINT)
						{
							if (pNM->hwndFrom == m_hLVMain)
							{
								if (m_ReceivedPackets[pCD->nmcd.dwItemSpec]->action == ACTION_NONE)
								{
									pCD->clrText = RGB(0, 0, 0);
								}
								else if (m_ReceivedPackets[pCD->nmcd.dwItemSpec]->action == ACTION_ACCESS_GRANTED)
								{
									pCD->clrText = RGB(80, 160, 0);
								}
								else if (m_ReceivedPackets[pCD->nmcd.dwItemSpec]->action == ACTION_ACCESS_DENIED)
								{
									pCD->clrText = RGB(255, 0, 0);
								}

								return CDRF_DODEFAULT;
							}
						}
					}
					break;

					case LVN_KEYDOWN:
					{
						LPNMLVKEYDOWN pLVKeyDown = (LPNMLVKEYDOWN)lParam;

						if (pLVKeyDown->wVKey != VK_DOWN &&
							pLVKeyDown->wVKey != VK_UP)
						{
							break;
						}

						nSelectedItem = ListView_GetSelectionMark(m_hLVMain);

						if (pLVKeyDown->wVKey == VK_UP &&
							nSelectedItem != 0)
						{
							nSelectedItem = -1;
						}
						else if(pLVKeyDown->wVKey == VK_DOWN &&
								nSelectedItem < (ListView_GetItemCount(m_hLVMain) - 1))
						{
							int nCount = ListView_GetItemCount(m_hLVMain);
							nSelectedItem = 1;
						}
						else
						{
							nSelectedItem = 0;
						}
					}
					// NOTICE: Fall through

					case NM_CLICK:
					{
						nSelectedItem += ListView_GetSelectionMark(m_hLVMain);

						if (nSelectedItem >= 0)
						{
							PacketData *pPacketData;
							pPacketData = m_ReceivedPackets[nSelectedItem];

							std::wstring hexDump;
							PrepareHexDump(
										hexDump,
										pPacketData->data,
										0,
										pPacketData->length,
										32);

							SetWindowText(m_hSTHexDump, hexDump.c_str());
							PopulateStatusBarData(pPacketData);
						}
					}
					break;
				}
			}
        }
        break;

		case WM_CTLCOLORSTATIC:
		{
			HDC hdc = (HDC) wParam;
			HWND hwndCtl = (HWND) lParam;

			if (hwndCtl == m_hSTHexDump)
			{
				HFONT hFont = CreateFont(	14, 0, 0, 0, FW_THIN,
											FALSE,FALSE,FALSE,
											ANSI_CHARSET,
											OUT_DEFAULT_PRECIS,
											CLIP_DEFAULT_PRECIS,
											DEFAULT_QUALITY,
											DEFAULT_PITCH | FF_SWISS,
											L"Courier New");

				SelectObject(hdc, hFont);
				SetTextColor(hdc, RGB(0, 0, 128));
			}

			return true;
		}
		break;

        case WM_QUERYNEWPALETTE:    // FALL THROUGH
        case WM_PALETTECHANGED:
        {
            /*
                invalidate to refresh palette
            */
            InvalidateRect(m_hSTHexDump, NULL, FALSE);

			return 0;
        }
        break;

        case WM_COMMAND:
        {
            /*
             * check for menu selection
             */
            if(	HIWORD(wParam) == 0 || HIWORD(wParam) == 1)
            {
                BOOL bHandled = TRUE;

                /*
                    MENU SELECTION
                */
                switch(LOWORD(wParam))
                {
                    case ID_FILE_EXIT:
                    {
						if (ghAdapter != INVALID_HANDLE_VALUE)
						{
							// Show error
							return 0;
						}

						return DefWindowProc(hwnd, WM_CLOSE, wParam, lParam);
                    }

					case ID_SNIFFER_ACTIVATE_SNIFFER:
					{
						RNIF_ADAPTER_INFO adapterInfo;
						if (InputAdapterName(adapterInfo))
						{
							if (ActivateSniffer(adapterInfo) > 0)
							{
								SetFilterDirected();
								m_SnifferActivated		= true;
								m_ProtectingMyComputer	= true;								
								UpdateMenus();
							}
						}

						break;
					}

					case ID_SNIFFER_DEACTIVATE_SNIFFER:
					{
						DeactivateSniffer();						
						m_SnifferActivated		= false;
						m_ProtectingMyComputer	= false;						
						UpdateMenus();						
						break;
					}

					case ID_SNIFFER_MYCOMPUTER:
					{
						SetFilterDirected();						
						m_ProtectingMyComputer	= true;						
						UpdateMenus();
						break;
					}

					case ID_SNIFFER_MYNETWORK:
					{
						SetFilterPromiscuous();
						m_ProtectingMyComputer	= false;						
						UpdateMenus();
						break;
					}

					case ID_FIREWALL_ACTIVATE_FIREWALL:
					{
						OPENFILENAME NameOfFile;
                        memset( &NameOfFile,
                                0,
                                sizeof(OPENFILENAME));

                        NameOfFile.lStructSize 	= sizeof(OPENFILENAME);
                        NameOfFile.hwndOwner 	= hwnd;

						wchar_t szACLFile[]	= L"All Files (*.*)\0*.*\0\0";
						wchar_t szFileName[MAX_PATH] = {0};
						
						NameOfFile.lpstrFilter = szACLFile;
                        NameOfFile.lpstrFile 	= szFileName;
						NameOfFile.nMaxFile		= MAX_PATH;
                        NameOfFile.Flags		= OFN_FILEMUSTEXIST | OFN_LONGNAMES;
						NameOfFile.lpstrTitle 	= L"Please select the Firewall Deny IP List File";

                        if (GetOpenFileName(&NameOfFile))
                        {
							ActivateFirewall(NameOfFile.lpstrFile);
							m_FirewallActivated = true;
							UpdateMenus();
                        }
						
						break;
					}
					
					case ID_FIREWALL_DEACTIVATE_FIREWALL:
					{
						m_FirewallActivated = false;
						UpdateMenus();
						break;
					}

                    default:
					{
                        bHandled = FALSE;
                        break;
					}
                }

                if (bHandled)
				{
                    return 0;
				}
            }
        }
        break;

        case WM_SIZE:
        {
            RECT rectClient;
            GetClientRect(  hwnd,
                            &rectClient);

            RECT rectClientStatusBar;
            GetClientRect(m_hwndStatus, &rectClientStatusBar);

			int listControlHeight = (((rectClient.bottom - rectClient.top - (rectClientStatusBar.bottom - rectClientStatusBar.top)) * 70) / 100);

			/*
			 * Windows control's size handling
			 */
            SetWindowPos(	m_hLVMain,
							NULL,
							0,
							0,
							rectClient.right - rectClient.left,
							listControlHeight,
							SWP_NOZORDER | SWP_NOACTIVATE | SWP_SHOWWINDOW);

			SetWindowPos(	m_hSTHexDump,
							NULL,
							1,
							listControlHeight,
							rectClient.right - rectClient.left,
							((rectClient.bottom - rectClient.top) - (rectClientStatusBar.bottom - rectClientStatusBar.top) - listControlHeight),
							SWP_NOZORDER | SWP_NOACTIVATE | SWP_SHOWWINDOW);

			InvalidateRect(m_hSTHexDump, NULL, TRUE);
			InvalidateRect(m_hLVMain, NULL, TRUE);
			UpdateWindow(m_hSTHexDump);
			UpdateWindow(m_hLVMain);

            /*
             * SetWindowPos does not work for status bar and this is
             * the method for its resizing
             */
            SendMessage(    m_hwndStatus,
                            WM_SIZE,
                            wParam,
                            lParam);
        }
        break;

        case WM_CHAR:
        {
            switch (wParam) 
            { 
                /* 
                 * Handling of escape key. 
				 * Issue-Issue!!! - This is leftover code
                 */
                case 0x1B:
                {                    
					return 0;
                }
            }

            break;
        }

		case WM_CLOSE:
        {
			if (ghAdapter != INVALID_HANDLE_VALUE)
			{
				// Firewall is active, you can't close
				return 0;
			}

			break;
        }

		case WM_CREATE:
		{
			/*
			 * Install and start timer
			 */
			SetTimer(	hwnd,
						SFGUI_REFRESH_TIMER_EVENT_ID,
						SFGUI_REFRESH_TIMER_DURATION,
						NULL);

			/*
			 * Install the driver dynamically and Load the driver
			 */
			RnIf_DriverInitInstance(L"RawNetGui");

			return 0;
		}

        case WM_DESTROY:
        {
			/*
			 * Unload the driver and uninstall it too if it was 
			 * dynamically installed
			 */
			RnIf_DriverExitInstance();

			// Save the window size information and post quit
            SaveAppInformation();
            PostQuitMessage(0);

            return 0;
        }

		case WM_TIMER:
		{
			if (wParam == SFGUI_REFRESH_TIMER_EVENT_ID)
			{
				if (m_ReceivedPackets.size() > m_NumListElements)
				{
					LVITEM  lvItem;
					ZeroMemory(&lvItem, sizeof(LVITEM));

					while (m_NumListElements < m_ReceivedPackets.size())
					{
						PacketData *pPacketData = m_ReceivedPackets[m_NumListElements];

						lvItem.mask		= LVIF_TEXT | LVIF_IMAGE;
						lvItem.iItem	= m_NumListElements;

						/*
						 * source MAC Address
						 */
						std::wstring sourceMAC;
						GetSourceMacAddress(sourceMAC, pPacketData);

						lvItem.iSubItem = SFGUI_SOURCE_MAC_ADDRESS_POS;
						lvItem.pszText	= (LPWSTR)sourceMAC.c_str();

						if (pPacketData->action == ACTION_NONE)
						{
							lvItem.iImage = SFGUI_IMAGE_BLACK;
						}
						else if (pPacketData->action == ACTION_ACCESS_GRANTED)
						{
							lvItem.iImage = SFGUI_IMAGE_GREEN;
						}
						else if (pPacketData->action == ACTION_ACCESS_DENIED)
						{
							lvItem.iImage = SFGUI_IMAGE_RED;
						}

						ListView_InsertItem(	m_hLVMain,
												&lvItem);

						/*
						 * Destination MAC Address
						 */
						std::wstring destMAC;
						GetDestMacAddress(destMAC, pPacketData);

						lvItem.iSubItem = SFGUI_DEST_MAC_ADDRESS_POS;
						lvItem.pszText	= (LPWSTR)destMAC.c_str();

						ListView_SetItem(   m_hLVMain,
											&lvItem);

						/*
						 * Protocol Name
						 */
						std::wstring datagramType;
						GetDatagramTypeString(datagramType, pPacketData);

						bool ipv4Datagram = IsIPV4Datagram(pPacketData);

						lvItem.iSubItem = SFGUI_DATAGRAM_NAME_POS;
						lvItem.pszText	= (LPWSTR)datagramType.c_str();

						ListView_SetItem(   m_hLVMain,
											&lvItem);

						if (ipv4Datagram)
						{
							/*
							* Source IP Address
							*/
							std::wstring sourceIP;
							GetSourceIP(sourceIP, pPacketData);

							lvItem.iSubItem = SFGUI_SOURCE_IP_ADDRESS_POS;
							lvItem.pszText	= (LPWSTR)sourceIP.c_str();

							ListView_SetItem(   m_hLVMain,
												&lvItem);

							/*
							* Destination IP Address
							*/
							std::wstring destIP;
							GetDestIP(destIP, pPacketData);

							lvItem.iSubItem = SFGUI_DEST_IP_ADDRESS_POS;
							lvItem.pszText	= (LPWSTR)destIP.c_str();

							ListView_SetItem(   m_hLVMain,
												&lvItem);
						}

						m_NumListElements++;
					}
				}
			}
			break;
		}

        default:
            break;
    }
    
	return DefWindowProc(hwnd,nMsg,wParam,lParam);
}



/****************************************************************************
 *  FUNCTION NAME:  InitInstance
 *
 *  DESCRIPTION:    Create the main window and other control
 *
 *  INPUTS:         
 *
 *  RETURNS:
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
CSfGuiView::InitInstance()
{
    CreateMainWindow();
    CreateMainWindowControl();
    CreateAndAttachImageList();
    CreateStatusBar();

    ShowWindow(m_hwnd, SW_SHOW);
    UpdateWindow(m_hwnd);

	SfInitialize(this);
}



/****************************************************************************
 *  FUNCTION NAME:  CreateMainWindow
 *
 *  DESCRIPTION:    Creates the main application window
 *
 *  INPUTS:         
 *
 *  RETURNS:
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
CSfGuiView::CreateMainWindow()
{
    static wchar_t szAppName[ ] = L"TCPFire";
    WNDCLASSEX wndclass;

    /*
        setup window class
    */
    wndclass.cbSize				= sizeof(wndclass);
    wndclass.style				= CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc		= ::WndProc;
    wndclass.cbClsExtra			= 0;
    wndclass.cbWndExtra			= 0;
    wndclass.hInstance			= m_hInstance;

    wndclass.hIcon			= LoadIcon( m_hInstance,
										(LPCTSTR)IDI_APP_ICON);

    wndclass.hCursor		= LoadCursor(	NULL,
											IDC_ARROW);

	wndclass.hIconSm		= LoadIcon(	m_hInstance,
										(LPCTSTR)IDI_APP_ICON);

    wndclass.hbrBackground	= (HBRUSH)GetStockObject(WHITE_BRUSH);
    wndclass.lpszMenuName	= (wchar_t*)IDR_MENU;
    wndclass.lpszClassName	= szAppName;
    

    RegisterClassEx(&wndclass);


    /*
        load if there are any previous window state saved for it
    */
    RECT rect;

	// Vague defaults
    rect.left	= 100;
    rect.right	= 800;
    rect.top	= 20;
    rect.bottom = 570;

    LoadAppInformation(&rect);


    /* 
        create main window
    */
    m_hwnd = CreateWindowEx(0,
                            szAppName,
                            SFGUI_APP_TITLE,
                            WS_OVERLAPPEDWINDOW,
                            rect.left,
                            rect.top,
                            rect.right - rect.left,
                            rect.bottom - rect.top,
                            NULL,
                            NULL,
                            m_hInstance,
                            NULL);

	/*
		check return value
	*/
	if (!m_hwnd)
	{
		return;
	}

	m_hAccel = LoadAccelerators(	m_hInstance,
									(LPCTSTR)IDR_MENU);

	UpdateMenus();
}


/****************************************************************************
 *  FUNCTION NAME:  UpdateMenus
 *
 *  DESCRIPTION:    Enable/Disable/Check/Uncheck all menus
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
CSfGuiView::UpdateMenus()
{
	/*
     * Initialize menus
     */
    HMENU hmenu = GetMenu(m_hwnd);

	if (m_SnifferActivated)
	{
		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_SNIFFER_MYCOMPUTER,
						(!m_ProtectingMyComputer) ? MF_ENABLED : MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_SNIFFER_MYNETWORK,
						(m_ProtectingMyComputer) ? MF_ENABLED : MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_SNIFFER_ACTIVATE_SNIFFER,
						MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_SNIFFER_DEACTIVATE_SNIFFER,
						MF_ENABLED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_FILE_EXIT,
						MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_FIREWALL_ACTIVATE_FIREWALL,
						(!m_FirewallActivated) ? MF_ENABLED : MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_FIREWALL_DEACTIVATE_FIREWALL,
						(m_FirewallActivated) ? MF_ENABLED : MF_GRAYED);
	}
	else
	{
		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_SNIFFER_MYCOMPUTER,
						MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_SNIFFER_MYNETWORK,
						MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_SNIFFER_ACTIVATE_SNIFFER,
						MF_ENABLED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_SNIFFER_DEACTIVATE_SNIFFER,
						MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_FILE_EXIT,
						MF_ENABLED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_FIREWALL_ACTIVATE_FIREWALL,
						MF_GRAYED);

		EnableMenuItem(	(struct HMENU__ *)hmenu,
						ID_FIREWALL_DEACTIVATE_FIREWALL,
						MF_GRAYED);
	}
}



/****************************************************************************
 *  FUNCTION NAME:  CreateMainWindowControl
 *
 *  DESCRIPTION:    Creates the List Control for packets and static control
 *					for Hex Dump
 *
 *  INPUTS:         
 *
 *  RETURNS:
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
CSfGuiView::CreateMainWindowControl()
{
    RECT rectClient;


    GetClientRect(m_hwnd,&rectClient);

	int listControlHeight = (((rectClient.bottom - rectClient.top) * 70) / 100);

    /*
        create and fill list view
    */
    m_hLVMain = CreateWindowEx( 0,
                                WC_LISTVIEW,
                                NULL,
                                WS_CHILD | WS_VISIBLE,
                                0,
                                0,
                                rectClient.right - rectClient.left,
                                listControlHeight,
                                m_hwnd,
                                NULL,
                                m_hInstance,
                                NULL);

    if (m_hLVMain)
    {
        /*
         * Set report style
         */
        SetWindowLong(  m_hLVMain,
                        GWL_STYLE,
                        GetWindowLong(  m_hLVMain,
                                        GWL_STYLE
                                    ) & (~(LVS_ICON | LVS_SMALLICON | LVS_LIST | LVS_REPORT)) | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SINGLESEL);

        ListView_SetExtendedListViewStyle(  m_hLVMain,
                                            LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);

        /*
         * Create Column
        */
        LVCOLUMN lvColumn;

        lvColumn.mask = LVCF_TEXT | LVCF_WIDTH;


        /*
         * Insert columns in main List control
         */
		for (	int listHeaderItem = 0;
				listHeaderItem < (sizeof(gMainListControlData) / sizeof(gMainListControlData[0]));
				++listHeaderItem)
		{
			lvColumn.cx			= gMainListControlData[listHeaderItem].size;
			lvColumn.pszText	= gMainListControlData[listHeaderItem].title;

			ListView_InsertColumn(  m_hLVMain,
									gMainListControlData[listHeaderItem].pos,
									&lvColumn);
		}

    }

    /*
     * Static text box for hex dump
     */
    m_hSTHexDump = CreateWindowEx(	0,
									L"edit",
									NULL,
									WS_CHILD | WS_VISIBLE | WS_BORDER | WS_HSCROLL | WS_VSCROLL | ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_MULTILINE | ES_READONLY,
									1,
									listControlHeight,
									rectClient.right - rectClient.left,
									((rectClient.bottom - rectClient.top) - listControlHeight),
									m_hwnd,
									NULL,
									m_hInstance,
									NULL);

    if (m_hSTHexDump)
    {
        /*
         * Set text box color and style
         */
    }
}



/****************************************************************************
 *  FUNCTION NAME:  CreateAndAttachImageList
 *
 *  DESCRIPTION:    Creates the Images to be associated with List Control
 *                  and attach them to the List control
 *
 *  INPUTS:         
 *
 *  RETURNS:
 *
 *  NOTES:          
 *
 ***************************************************************************/
void 
CSfGuiView::CreateAndAttachImageList()
{
    HBITMAP hBitmap;

    /*
     * Create image lists as dib section 24-bit color
     * large image list
     */
    m_hILLarge = ImageList_Create(  32,
                                    32,
                                    ILC_COLOR24 | ILC_MASK,
                                    9,
                                    1);

    hBitmap = (struct HBITMAP__ *)LoadImage(    m_hInstance,
                                                MAKEINTRESOURCE(IDB_RED_SIGN),
                                                IMAGE_BITMAP,
                                                0,
                                                0,
                                                LR_CREATEDIBSECTION);

    ImageList_AddMasked(    m_hILLarge,
                            hBitmap,
                            RGB(0,255,255));
    DeleteObject(hBitmap);

    /*
        SMALL IMAGE LIST
    */
    m_hILSmall = ImageList_Create(  16,
                                    16,
                                    ILC_COLOR24 | ILC_MASK,
                                    4,
                                    1);


    hBitmap = (struct HBITMAP__ *)LoadImage(    m_hInstance,
                                                MAKEINTRESOURCE(IDB_RED_SIGN_SMALL),
                                                IMAGE_BITMAP,
                                                0,
                                                0,
                                                LR_CREATEDIBSECTION);

    ImageList_AddMasked(    m_hILSmall,
                            hBitmap,
                            RGB(0,255,255));
    DeleteObject(hBitmap);


    hBitmap = (struct HBITMAP__ *)LoadImage(    m_hInstance,
                                                MAKEINTRESOURCE(IDB_GREEN_SIGN_SMALL),
                                                IMAGE_BITMAP,
                                                0,
                                                0,
                                                LR_CREATEDIBSECTION);
    ImageList_AddMasked(    m_hILSmall,
                            hBitmap,
                            RGB(0,255,255));
    DeleteObject(hBitmap);


    hBitmap = (struct HBITMAP__ *)LoadImage(    m_hInstance,
                                                MAKEINTRESOURCE(IDB_BLUE_SIGN_SMALL),
                                                IMAGE_BITMAP,
                                                0,
                                                0,
                                                LR_CREATEDIBSECTION);

    ImageList_AddMasked(    m_hILSmall,
                            hBitmap,
                            RGB(0,255,255));
    DeleteObject(hBitmap);


    hBitmap = (struct HBITMAP__ *)LoadImage(    m_hInstance,
                                                MAKEINTRESOURCE(IDB_BLACK_SIGN_SMALL),
                                                IMAGE_BITMAP,
                                                0,
                                                0,
                                                LR_CREATEDIBSECTION);
    ImageList_AddMasked(    m_hILSmall,
                            hBitmap,
                            RGB(0,255,255));
    DeleteObject(hBitmap);

    /* 
     * Assign image lists to list controls
     */
    ListView_SetImageList(  m_hLVMain,
                            m_hILLarge,
                            LVSIL_NORMAL);

    ListView_SetImageList(  m_hLVMain,
                            m_hILSmall,
                            LVSIL_SMALL);
}



/****************************************************************************
 *  FUNCTION NAME:  CreateStatusBar
 *
 *  DESCRIPTION:
 *
 *  INPUTS:         
 *
 *  RETURNS:
 *
 *  NOTES:          
 *
 ***************************************************************************/
int
CSfGuiView::CreateStatusBar()
{
    RECT rectClient;
    int lpParts[SFGUI_NUM_STATUS_PARTS];
    int i, nWidth;

    /*
        CREATE THE STATUS BAR.
    */
    m_hwndStatus = CreateWindowEx( 
                    0,                          // no extended styles
                    STATUSCLASSNAME,            // name of status bar class
                    (LPCTSTR) NULL,             // no text when first created
                    SBARS_SIZEGRIP |            // includes a sizing grip
                    WS_CHILD | WS_VISIBLE,      // creates a child window
                    0, 0, 0, 0,                 // ignores size and position
                    m_hwnd,                     // handle to parent window
                    NULL,						// child window identifier
                    m_hInstance,                // handle to application instance
                    NULL);                      // no window creation data


    GetClientRect(m_hwnd, &rectClient);

    /*
        calculate the right edge coordinate for each part, and
        copy the coordinates to the array.
    */
    nWidth = SFGUI_WIDTH_OF_FIRST_PART;
	lpParts[0] = nWidth;

	nWidth += SFGUI_WIDTH_OF_SECOND_PART;
	lpParts[1] = nWidth;

	nWidth += SFGUI_WIDTH_OF_THIRD_PART;
	lpParts[2] = nWidth;

	nWidth += SFGUI_WIDTH_OF_FOURTH_PART;
	lpParts[3] = nWidth;

    /*
        Tell the status bar to create the window parts.
    */
    SendMessage(    m_hwndStatus,
                    SB_SETPARTS,
                    (WPARAM) SFGUI_NUM_STATUS_PARTS,
                    (LPARAM) lpParts);

    SendMessage(    m_hwndStatus,
                    SB_SETTEXT,
                    (WPARAM) 0,	// | SBT_POPOUT | SBT_NOBORDERS,
					(LPARAM) L"Packet Size:");

	SendMessage(    m_hwndStatus,
                    SB_SETTEXT,
                    (WPARAM) 2,
					(LPARAM) L"Protocol:");

    return 0;
}


/****************************************************************************
 *  FUNCTION NAME:  PopulateStatusBarData
 *
 *  DESCRIPTION:    Shows the %age match of the files in Status bar
 *
 *  INPUTS:         
 *
 *  RETURNS:
 *
 *  NOTES:          
 *
 ***************************************************************************/
void
CSfGuiView::PopulateStatusBarData(PacketData *inPacketData)
{
    wchar_t szText[16];

    wsprintf(	szText,
				L"%d bytes",
				inPacketData->length);

    SendMessage(	m_hwndStatus,
					SB_SETTEXT,
					(WPARAM) 1,
					(LPARAM) szText);

	std::wstring protocolName;

	GetEthernetProtocolName(protocolName, inPacketData);

    SendMessage(	m_hwndStatus,
					SB_SETTEXT,
					(WPARAM) 3,
					(LPARAM) protocolName.c_str());
}


/****************************************************************************
 *  FUNCTION NAME:  GetAccel
 *
 *  DESCRIPTION:    Returns Handle to Loaded Accelerator
 *
 *  NOTES:          
 *
 ***************************************************************************/
HACCEL
CSfGuiView::GetAccel()
{
	return m_hAccel;
}



/****************************************************************************
 *  FUNCTION NAME:  SaveAppInformation
 *
 *  DESCRIPTION:    
 *
 *  NOTES:          
 *
 ***************************************************************************/
int
CSfGuiView::SaveAppInformation()
{
    RECT rectMain;
    int nRetVal = 0;
    
    if (GetWindowRect(  m_hwnd,
                        &rectMain))
    {
        HKEY hKey;
	    wchar_t szKey[] = L"Software\\Microsoft\\TCPFire";

	    nRetVal = RegCreateKey(	HKEY_CURRENT_USER,
							    szKey,
							    &hKey);

	    if (nRetVal != ERROR_SUCCESS)
	    {
		    return -1;
	    }

	    nRetVal = RegSetValueEx(	hKey,
								    L"X1",
								    0,
								    REG_DWORD,
								    (const unsigned char *)&rectMain.left,
								    sizeof(rectMain.left));

        if (nRetVal != ERROR_SUCCESS)
	    {
            RegCloseKey(hKey);
		    return -1;
	    }


        nRetVal = RegSetValueEx(	hKey,
								    L"Y1",
								    0,
								    REG_DWORD,
								    (const unsigned char *)&rectMain.top,
								    sizeof(rectMain.top));

        if (nRetVal != ERROR_SUCCESS)
	    {
            RegCloseKey(hKey);
		    return -1;
	    }


        nRetVal = RegSetValueEx(	hKey,
								    L"X2",
								    0,
								    REG_DWORD,
								    (const unsigned char *)&rectMain.right,
								    sizeof(rectMain.right));

        if (nRetVal != ERROR_SUCCESS)
	    {
            RegCloseKey(hKey);
		    return -1;
	    }

        nRetVal = RegSetValueEx(	hKey,
								    L"Y2",
								    0,
								    REG_DWORD,
								    (const unsigned char *)&rectMain.bottom,
								    sizeof(rectMain.bottom));

        if (nRetVal != ERROR_SUCCESS)
	    {
            RegCloseKey(hKey);
		    return -1;
	    }

        RegCloseKey(hKey);
    }

    return 0;
}



/****************************************************************************
 *  FUNCTION NAME:  LoadAppInformation
 *
 *  DESCRIPTION:    
 *
 *  NOTES:          
 *
 ***************************************************************************/
int
CSfGuiView::LoadAppInformation(RECT *pRect)
{
    int nRetVal = 0;
    
    HKEY hKey;
    unsigned long nDataSize = 0;

	wchar_t szKey[] = L"Software\\Microsoft\\TCPFire";

	nRetVal = RegOpenKey(	HKEY_CURRENT_USER,
							szKey,
							&hKey);

	if (nRetVal != ERROR_SUCCESS)
	{
		return -1;
	}

    nDataSize = sizeof(pRect->left);
	nRetVal = RegQueryValueEx(	hKey,
								L"X1",
								0,
								NULL,
								(unsigned char *)&pRect->left,
								&nDataSize);

    if (nRetVal != ERROR_SUCCESS)
	{
        RegCloseKey(hKey);
		return -1;
	}


    nDataSize = sizeof(pRect->top);
    nRetVal = RegQueryValueEx(	hKey,
								L"Y1",
								0,
								NULL,
								(unsigned char *)&pRect->top,
								&nDataSize);

    if (nRetVal != ERROR_SUCCESS)
	{
        RegCloseKey(hKey);
		return -1;
	}


    nDataSize = sizeof(pRect->right);
    nRetVal = RegQueryValueEx(	hKey,
								L"X2",
								0,
								NULL,
								(unsigned char *)&pRect->right,
								&nDataSize);

    if (nRetVal != ERROR_SUCCESS)
	{
        RegCloseKey(hKey);
		return -1;
	}

    nDataSize = sizeof(pRect->bottom);
    nRetVal = RegQueryValueEx(	hKey,
								L"Y2",
								0,
								NULL,
								(unsigned char *)&pRect->bottom,
								&nDataSize);

    if (nRetVal != ERROR_SUCCESS)
	{
        RegCloseKey(hKey);
		return -1;
	}
    RegCloseKey(hKey);

    return 0;
}


/****************************************************************************
 *  FUNCTION NAME:  InputAdapterName
 *
 *  DESCRIPTION:    Asks the user for NDIS adapter to select
 *
 *  NOTES:          
 *
 ***************************************************************************/
bool
CSfGuiView::InputAdapterName(RNIF_ADAPTER_INFO &adapterInfo)
{
	if (RnIf_DriverIsRunning())
	{
		RNIF_ADAPTER_INFO adapterArray[64];
		RNIF_ADAPTER_INFO_LIST adapterInfoList;
		adapterInfoList.AdapterArray = adapterArray;
		adapterInfoList.MaximumCount = 64;
		adapterInfoList.CurrentCount = 0;
		RnIf_AdapterEnumerate(adapterInfoList);

		int numAdapters = adapterInfoList.CurrentCount;

		/*
		 * Popup a dialog box here and ask user to select
		 * the name of adapter to apply firewall on
		 */
		int nResult = 0;
		nResult = (INT)DialogBoxParam(	m_hInstance,
										MAKEINTRESOURCE(IDD_ADAPTER_LIST),
										m_hwnd,
										(DLGPROC)AdapterListDlgProc,
										(LPARAM)&adapterInfoList);

		if (nResult < 0)
		{
			return false;
		}

		adapterInfo = adapterInfoList.AdapterArray[nResult];
	}

	return true;
}



/****************************************************************************
 *  FUNCTION NAME:  HandleNewPacket
 *
 *  DESCRIPTION:    Process the new packet received from the network
 *
 *  NOTES:
 *
 ***************************************************************************/
void
CSfGuiView::HandleNewPacket(PacketData *pPacketData)
{
	m_ReceivedPackets.push_back(pPacketData);
}



/****************************************************************************
 *  FUNCTION NAME:  AdapterListDlgProc
 *
 *  DESCRIPTION:    Shows a list of adapters in a modal dialog box
 *
 *  NOTES:          
 *
 ***************************************************************************/
INT_PTR CALLBACK
AdapterListDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	HWND	hListCtrl;
	int		result = 0;

	switch(msg)
	{
		case WM_INITDIALOG:
		{
			hListCtrl = GetDlgItem(hDlg, IDC_ADAPTER_LIST);

			/*
			 * Set List Control Style to Full row select
			 */
			ListView_SetExtendedListViewStyle(  hListCtrl,
												LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);

			/*
			 * Insert Column in List
			 */
			LVCOLUMN lvColumn;
			lvColumn.mask = LVCF_TEXT | LVCF_WIDTH;

			lvColumn.cx = 453;
			lvColumn.pszText = L"Adapter Name";
			ListView_InsertColumn(  hListCtrl,
									0,
									&lvColumn);

			RNIF_ADAPTER_INFO_LIST *adapterInfoList = 
											(RNIF_ADAPTER_INFO_LIST *)lParam;

			hListCtrl = GetDlgItem(hDlg, IDC_ADAPTER_LIST);
			ListView_DeleteAllItems(hListCtrl);

			int numAdapters = adapterInfoList->CurrentCount;
			for (int i = 0; i < numAdapters; ++i)
			{
				LVITEM  lvItem;
				ZeroMemory(&lvItem, sizeof(LVITEM));

				lvItem.mask		= LVIF_TEXT | LVIF_IMAGE;
				lvItem.iItem	= i;
				lvItem.iSubItem = 0;
				lvItem.pszText	= adapterInfoList->AdapterArray[i].FriendlyName;
				lvItem.iImage	= SFGUI_IMAGE_RED;

				ListView_InsertItem(	hListCtrl,
										&lvItem);
			}

			break;
		}
		case WM_COMMAND:
		{
			switch(wParam)
			{
				case IDOK:
				{
					hListCtrl	= GetDlgItem(hDlg, IDC_ADAPTER_LIST);
					result		= ListView_GetSelectionMark(hListCtrl);
					EndDialog(hDlg, result);
                    return TRUE;
				}
				case IDCANCEL:
				{
					EndDialog(hDlg, -1);
                    return TRUE;
				}
			}

			break;
		}
	}

	return FALSE;
}


